import logging
import sys

log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

console_handler = logging.StreamHandler(sys.stdout)
console_handler.setFormatter(logging.Formatter(log_format))

logger = logging.getLogger()

logger.setLevel(logging.INFO)

# Remove existing handlers to avoid duplicate logs
if logger.hasHandlers():
    logger.handlers.clear()

# Add the console handler to the root logger
logger.addHandler(console_handler)

# Example of how to get a specific logger in other modules


def get_logger(name: str) -> logging.Logger:
    return logging.getLogger(name)


print("Logger configured.")



from starlette.applications import Starlette
from starlette.requests import Request

from starlette.routing import Mount, Host, Route
from mcp.server.fastmcp import Context, FastMCP
from mcp.server.sse import SseServerTransport
from starlette.types import Receive, Scope, Send
from auth import JwtAuthTransport
import uvicorn
import os
from dotenv import load_dotenv
from configs.ragflow import ragflow
from services.chat_assistant import ask_ragflow, create_chat_session
from services.dataset import create_initial_dataset, get_dataset_by_name
from settings import settings
import json
from global_session import user_sessions
import asyncio

load_dotenv()


def get_transport():
    try:
        if settings.enable_auth:
            return JwtAuthTransport("/messages/")
        return SseServerTransport("/messages/")
    except Exception as e:
        print(f"Warning: Error initializing transport: {e}")
        return SseServerTransport("/messages/")  # Fallback to SSE transport


mcp = FastMCP("Ragflow MCP")
transport = get_transport()


async def handle_sse(request):
    try:
        async with transport.connect_sse(
            request.scope, request.receive, request._send
        ) as streams:
            await mcp._mcp_server.run(
                streams[0],
                streams[1],
                mcp._mcp_server.create_initialization_options(),
            )
    except Exception as e:
        print(f"Error in handle_sse: {e}")
        raise


async def wrap_handle_post_message(scope: Scope,
                                   receive: Receive,
                                   send: Send):
    temp_request = Request(scope, receive)
    authorization = temp_request.headers.get("authorization")
    # ---- Start modify body ----
    original_body_bytes = b''
    original_receive = receive
    more_body = True
    while more_body:
        message = await original_receive()
        print(f"Received message: {message}")
        if message['type'] == 'http.request':
            original_body_bytes += message.get('body', b'')
            more_body = message.get('more_body', False)
        elif message['type'] == 'http.disconnect':
            print("Client disconnected while reading body")
            return
        else:
            pass

    modified_body_bytes = original_body_bytes
    try:
        # Parse JSON
        data = json.loads(original_body_bytes.decode('utf-8'))
        if 'params' in data and 'arguments' in data['params']:
            if settings.enable_auth:
                data['params']['arguments']['dataset_name'] = authorization
        modified_body_bytes = json.dumps(data).encode('utf-8')
    except json.JSONDecodeError as e:
        print(f"Warning: Could not parse request body as JSON: {e}")

    except Exception as e:
        print(f"Error during body modification: {e}")
    _receive_called = False

    async def modified_receive():
        nonlocal _receive_called
        if not _receive_called:
            _receive_called = True
            return {'type': 'http.request', 'body': modified_body_bytes, 'more_body': False}
        else:
            await asyncio.sleep(3600)
            return {'type': 'http.disconnect'}

    return await transport.handle_post_message(scope, modified_receive, send)

app = Starlette(
    routes=[
        Route('/sse/', endpoint=handle_sse),
        Mount("/messages/", app=wrap_handle_post_message)
    ]
)


@mcp.tool()
def get_ragflow_datasets() -> str:
    try:
        datasets = ragflow.list_datasets()
        return datasets
    except Exception as e:
        return f"Error fetching datasets: {str(e)}"


@mcp.tool()
def create_rag(name: str) -> str:
    """Creates a initial knowledge base and dataset for the user.

    Args:
        name (str): The name of the dataset to create,

    Returns:
        str: Response from the API indicating success or failure
    """
    existed_datasets = get_dataset_by_name(name)
    if len(existed_datasets) > 0:
        return f"Dataset '{name}' already exists"
    try:
        response = create_initial_dataset(name)
        return f"Successfully created dataset '{name}': {response.id}"
    except Exception as e:
        return f"Failed to create dataset: {str(e)}"


@mcp.tool()
def upload_rag(dataset_name: str, display_names: list[str], blobs: list[str]) -> str:
    """Uploads documents and provide more knowledge base for the dataset.

    Args:
        dataset_name (str): The name of the dataset to upload documents to
        display_names (list[str]): List of display names for the documents
        blobs (list[str]): List of document contents as strings

    Returns:
        str: Response from the API indicating success or failure
    """
    try:
        print("dataset_name", dataset_name)
        dataset = ragflow.get_dataset(name=dataset_name)

        documents = []
        for display_name, blob in zip(display_names, blobs):
            documents.append({
                "display_name": display_name,
                "blob": blob
            })

        # Upload documents
        response = dataset.upload_documents(documents)

        # Get document IDs
        doc_info = []
        for doc in response:
            dataset.async_parse_documents([doc.id])
            doc_info.append({
                "name": doc.display_name if hasattr(doc, 'display_name') else display_names[0],
                "id": doc.id
            })

        # training

        return {
            "status": "success",
            "message": f"Successfully uploaded {len(documents)} documents",
            "dataset": dataset_name,
            "documents": doc_info
        }
    except Exception as e:
        return {
            "status": "error",
            "message": str(e)
        }


@mcp.tool()
def query_rag(dataset_name: str, query: str) -> str:
    """
    Queries the specified dataset in the knowledge base to retrieve an answer based on the provided query.

    Args:
        dataset_name (str): The name of the dataset to query.
        query (str): The question or query string to search for in the dataset.

    Returns:
        str: A dictionary containing:
            - "reference": Details of the reference data from the knowledge base, including content and source information.
            - "answer": The answer derived from the knowledge base. If the answer is not found, a message indicating this will be returned.

    Raises:
        Exception: If an error occurs during the query process, an error message is returned.
    """
    try:
        response = ask_ragflow(dataset_name, query)
        return {
            "reference": response['data']['reference'],
            "answer": response['data']['answer']
        }
    
    except Exception as e:
        return f"Error querying dataset: {str(e)}"


# or dynamically mount as host
app.router.routes.append(Host('mcp.acme.corp', app=app))

if __name__ == "__main__":
    # Run the server with Uvicorn

    uvicorn.run(app, host="0.0.0.0", port=8000)



import asyncio
import click
from mcp.client.session import ClientSession

from mcp.client.sse import sse_client


async def __main(endpoint: str):
    async with sse_client(
        url=endpoint,
        headers={"Authorization": "Bearer test-user-id"},
    ) as (read, write):
        async with ClientSession(read, write) as session:
            await session.initialize()

            # List available tools
            tools = await session.list_tools()
            print(tools)

            # Call the execute_command tool
            result = await session.call_tool("hello", {})
            print("-" * 100)
            print(result)


@click.command()
@click.option("--endpoint", default="http://localhost:8000/sse", help="URL of the SSE endpoint")
def main(endpoint: str):
    asyncio.run(__main(endpoint))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
RAGFlow MCP Server - stdio版本
为本地测试和打包部署提供stdio协议支持
"""
import asyncio
from mcp.server import Server
from mcp.server.stdio import stdio_server
from mcp.types import Tool, TextContent
from dotenv import load_dotenv
from configs.ragflow import ragflow
from services.chat_assistant import ask_ragflow, create_chat_session
from services.dataset import create_initial_dataset, get_dataset_by_name
import json

load_dotenv()

# 创建MCP Server实例（stdio模式）
server = Server("ragflow-mcp")


@server.list_tools()
async def list_tools() -> list[Tool]:
    """列出可用的工具"""
    return [
        Tool(
            name="get_ragflow_datasets",
            description="获取RAGFlow数据集列表",
            inputSchema={
                "type": "object",
                "properties": {},
                "required": []
            }
        ),
        Tool(
            name="create_rag",
            description="创建初始知识库和数据集",
            inputSchema={
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "description": "要创建的数据集名称"
                    }
                },
                "required": ["name"]
            }
        ),
        Tool(
            name="upload_rag",
            description="上传文档以增强数据集的知识库",
            inputSchema={
                "type": "object",
                "properties": {
                    "dataset_name": {
                        "type": "string",
                        "description": "要上传文档的数据集名称"
                    },
                    "display_names": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "文档的显示名称列表"
                    },
                    "blobs": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "文档内容字符串列表"
                    }
                },
                "required": ["dataset_name", "display_names", "blobs"]
            }
        ),
        Tool(
            name="query_rag",
            description="查询知识库中指定的数据集，基于提供的查询检索答案",
            inputSchema={
                "type": "object",
                "properties": {
                    "dataset_name": {
                        "type": "string",
                        "description": "要查询的数据集名称"
                    },
                    "query": {
                        "type": "string",
                        "description": "要在数据集中搜索的问题或查询字符串"
                    }
                },
                "required": ["dataset_name", "query"]
            }
        )
    ]


@server.call_tool()
async def call_tool(name: str, arguments: dict) -> list[TextContent]:
    """调用工具"""
    try:
        if name == "get_ragflow_datasets":
            result = ragflow.list_datasets()
            return [TextContent(type="text", text=str(result))]

        elif name == "create_rag":
            dataset_name = arguments.get("name")
            existed_datasets = get_dataset_by_name(dataset_name)
            if len(existed_datasets) > 0:
                result = f"Dataset '{dataset_name}' already exists"
            else:
                response = create_initial_dataset(dataset_name)
                result = f"Successfully created dataset '{dataset_name}': {response.id}"
            return [TextContent(type="text", text=result)]

        elif name == "upload_rag":
            dataset_name = arguments.get("dataset_name")
            display_names = arguments.get("display_names", [])
            blobs = arguments.get("blobs", [])

            dataset = ragflow.get_dataset(name=dataset_name)

            documents = []
            for display_name, blob in zip(display_names, blobs):
                documents.append({
                    "display_name": display_name,
                    "blob": blob
                })

            response = dataset.upload_documents(documents)

            doc_info = []
            for doc in response:
                dataset.async_parse_documents([doc.id])
                doc_info.append({
                    "name": doc.display_name if hasattr(doc, 'display_name') else display_names[0],
                    "id": doc.id
                })

            result = json.dumps({
                "status": "success",
                "message": f"Successfully uploaded {len(documents)} documents",
                "dataset": dataset_name,
                "documents": doc_info
            }, ensure_ascii=False)
            return [TextContent(type="text", text=result)]

        elif name == "query_rag":
            dataset_name = arguments.get("dataset_name")
            query = arguments.get("query")

            response = ask_ragflow(dataset_name, query)
            result = json.dumps({
                "reference": response['data']['reference'],
                "answer": response['data']['answer']
            }, ensure_ascii=False)
            return [TextContent(type="text", text=result)]

        else:
            return [TextContent(type="text", text=f"Unknown tool: {name}")]

    except Exception as e:
        return [TextContent(type="text", text=f"Error: {str(e)}")]


async def main():
    """主函数入口"""
    async with stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream,
            write_stream,
            server.create_initialization_options()
        )


if __name__ == "__main__":
    asyncio.run(main())


# app/config.py
import os
from dotenv import load_dotenv
from pydantic_settings import BaseSettings

# Load environment variables from .env file
load_dotenv()


class Settings(BaseSettings):
    """Application settings."""
    ragflow_api_key: str = os.getenv("RAGFLOW_API_KEY", "")
    ragflow_base_url: str = os.getenv(
        "RAGFLOW_BASE_URL", "")
    model_name: str = os.getenv("MODEL_NAME", ""),
    enable_auth: bool = os.getenv("AUTH_ENABLED", "").lower() == "true"

    class Config:
        # If you don't use .env file, pydantic can read directly from environment variables
        env_file = '.env'
        env_file_encoding = 'utf-8'
        extra = 'ignore'  # Ignore extra fields from environment


settings = Settings()
# Example usage (optional, just for demonstration)
if __name__ == "__main__":
    print(f"RAGFLOW API Key: {settings.ragflow_api_key}")
    print(f"RAGFLOW Base URL: {settings.ragflow_base_url}")



import base64
from contextlib import asynccontextmanager

from starlette.types import Scope, Receive, Send
from starlette.exceptions import HTTPException

from mcp.server.sse import SseServerTransport


class JwtAuthTransport(SseServerTransport):
    """
    Example basic auth implementation of SSE server transport.
    """

    def __init__(self, endpoint: str):
        super().__init__(endpoint)

    @asynccontextmanager
    async def connect_sse(self, scope: Scope, receive: Receive, send: Send):
        auth_header = dict(scope["headers"]).get(b'authorization', b'')
        print("auth_header", auth_header)
        if not auth_header:
            raise HTTPException(status_code=401, detail="Unauthorized")

        # global_session[auth_header] = {}
        async with super().connect_sse(scope, receive, send) as streams:
            # get session from global_session
            print("global_session", streams)
            yield streams



import json
from configs.ragflow import ragflow
from configs.logger import get_logger

logger = get_logger(__name__)


def get_dataset_by_name(user_id: str):
    try:
        datasets = ragflow.list_datasets(name=user_id)
        return datasets
    except Exception as e:
        logger.error(f"Error getting dataset by name {user_id}: {e}")
        return []


def create_initial_dataset(user_id: str):
    existed_datas = get_dataset_by_name(user_id)
    if len(existed_datas) > 0:
        return existed_datas[0]
    dataset = ragflow.create_dataset(name=user_id)
    documents = dataset.upload_documents([{
        "display_name": "base_knowledge.txt",
        "blob": "You are a helpful assistant that can answer questions about the user's data."
    }])
    document_ids = [doc.id for doc in documents]
    dataset.async_parse_documents(document_ids)
    return dataset


def upload_documents_to_dataset(dataset_name: str, display_names: list[str], blobs: list[str]) -> str:
    """Uploads documents to a RAGFlow dataset.

    Args:
        dataset_name (str): The name of the dataset to upload documents to
        display_names (list[str]): List of display names for the documents
        blobs (list[str]): List of document contents as strings

    Returns:
        str: Response from the API indicating success or failure
    """
    try:
        # Get the dataset object using dataset name
        dataset = ragflow.get_dataset(name=dataset_name)

        # Prepare documents list
        documents = []
        for display_name, blob in zip(display_names, blobs):
            documents.append({
                "display_name": display_name,
                "blob": blob
            })

        # Upload documents
        response = dataset.upload_documents(documents)

        # Get document IDs
        doc_info = []
        for doc in response:
            doc_info.append({
                "name": doc.display_name if hasattr(doc, 'display_name') else display_names[0],
                "id": doc.id
            })

        return {
            "status": "success",
            "message": f"Successfully uploaded {len(documents)} documents",
            "dataset": dataset_name,
            "documents": doc_info
        }
    except Exception as e:
        return {
            "status": "error",
            "message": str(e)
        }


def list_datasets(
    page: int = 1,
    page_size: int = 30,
    orderby: str = "create_time",
    desc: bool = True,
    name: str = None,
    dataset_id: str = None
) -> str:
    """Lists datasets with filtering options.

    Args:
        page (int, optional): Page number for pagination. Defaults to 1.
        page_size (int, optional): Number of items per page. Defaults to 30.
        orderby (str, optional): Field to sort by ('create_time' or 'update_time'). Defaults to 'create_time'.
        desc (bool, optional): Sort in descending order if True. Defaults to True.
        name (str, optional): Filter by dataset name. Defaults to None.
        dataset_id (str, optional): Filter by dataset ID. Defaults to None.

    Returns:
        str: List of datasets matching the criteria
    """
    try:
        # Validate orderby parameter
        if orderby not in ["create_time", "update_time"]:
            return "Error: orderby must be either 'create_time' or 'update_time'"

        # Build query parameters
        params = {
            "page": page,
            "page_size": page_size,
            "orderby": orderby,
            "desc": desc
        }

        # Add optional filters if provided
        if name:
            params["name"] = name
        if dataset_id:
            params["id"] = dataset_id

        # Get datasets with filters
        datasets = ragflow.list_datasets(**params)
        return f"Successfully retrieved datasets: {datasets}"
    except Exception as e:
        return f"Failed to list datasets: {str(e)}"


def list_documents_in_dataset(dataset_name: str, keywords: str = None) -> str:
    """Lists documents in a RAGFlow dataset with optional keyword filtering.

    Args:
        dataset_name (str): The name of the dataset to list documents from
        keywords (str, optional): Keywords to filter documents. Defaults to None.

    Returns:
        str: List of documents matching the criteria
    """
    try:
        # Get the dataset object
        dataset = ragflow.get_dataset(name=dataset_name)

        # List documents with optional keyword filter
        documents = dataset.list_documents(keywords=keywords)

        # Format document information
        doc_list = []
        for doc in documents:
            doc_list.append({
                "id": doc.id,
                "name": doc.display_name if hasattr(doc, 'display_name') else "Unknown",
                "status": "listed"
            })

        return {
            "status": "success",
            "message": f"Found {len(doc_list)} documents",
            "dataset": dataset_name,
            "documents": doc_list
        }
    except Exception as e:
        return {
            "status": "error",
            "message": str(e)
        }


def parse_documents_in_dataset(dataset_name: str, document_ids: list[str]) -> str:
    """Initiates async parsing of documents in a RAGFlow dataset.

    Args:
        dataset_name (str): The name of the dataset containing the documents
        document_ids (list[str]): List of document IDs to parse

    Returns:
        str: Response indicating parsing initiation status
    """
    try:
        # Get the dataset object
        dataset = ragflow.get_dataset(name=dataset_name)

        # Initiate async parsing
        dataset.async_parse_documents(document_ids)

        return {
            "status": "success",
            "message": "Async document parsing initiated",
            "dataset": dataset_name,
            "document_count": len(document_ids),
            "document_ids": document_ids
        }
    except Exception as e:
        return {
            "status": "error",
            "message": str(e)
        }


from datetime import datetime
from configs.ragflow import ragflow
from services.dataset import create_initial_dataset, get_dataset_by_name
from settings import settings
from configs.logger import get_logger
import json

logger = get_logger(__name__)


def create_chat_assistant(user_id: str):
    datasets = get_dataset_by_name(user_id)
    if len(datasets) == 0:
        datasets = [create_initial_dataset(user_id)]
    return ragflow.create_chat(user_id)


def get_chat_assistant(user_id: str):
    chats = []
    try:
        chats = ragflow.list_chats(name=user_id)
    except Exception as e:
        chats = []
    if len(chats) > 0:
        return chats[0]
    else:
        return create_chat_assistant(user_id)


def create_chat_session(user_id: str):
    chat = get_chat_assistant(user_id)
    if not chat:
        return None
    session_name = f"New session {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
    return chat.create_session(session_name)


def get_chat_session(user_id: str):
    sessions = []
    try:
        chat = get_chat_assistant(user_id)
        sessions = chat.list_sessions()
    except Exception as e:
        logger.error(f"Error getting chat session: {e}")
        return []
    if len(sessions) > 0:
        return sessions[0]
    else:
        return create_chat_session(user_id)


def ask_ragflow(user_id: str, question: str, stream: bool = False):
    session = create_chat_session(user_id)
    if not session:
        return {
            "data": {
                "reference": [],
                "answer": "Mock response: This is a test response for local testing"
            }
        }

    # Mock response for local testing
    return {
        "data": {
            "reference": [],
            "answer": "Mock response: This is a test response for local testing"
        }
    }


from settings import settings

# Mock RAGFlow for local testing
class MockRAGFlow:
    def __init__(self, api_key, base_url):
        self.api_key = api_key
        self.base_url = base_url

    def list_datasets(self, **kwargs):
        return "[]"

    def get_dataset(self, name):
        return MockDataset(name)

    def create_dataset(self, name):
        return MockDataset(name)

    def create_chat(self, name, **kwargs):
        return MockChat(name)

    def list_chats(self, name=None):
        return []

class MockDataset:
    def __init__(self, name):
        self.name = name
        self.id = f"dataset_{name}"

    def upload_documents(self, documents):
        class MockDoc:
            def __init__(self, idx):
                self.id = f"doc_{idx}"
                self.display_name = f"document_{idx}"

        return [MockDoc(i) for i in range(len(documents))]

    def async_parse_documents(self, doc_ids):
        pass

class MockChat:
    def __init__(self, name):
        self.name = name
        self.id = f"chat_{name}"

    def create_session(self, name):
        class MockSession:
            def __init__(self, name):
                self.name = name
                self.id = f"session_{name}"
                self.chat_id = f"chat_{name}"
        return MockSession(name)

    def list_sessions(self):
        return []

ragflow = MockRAGFlow(
    api_key=settings.ragflow_api_key,
    base_url=settings.ragflow_base_url
)


from setuptools import setup, find_packages

# 手动添加所有必要的模块和数据
py_modules = [
    'stdio_server', 'auth', 'client', 'global_session', 'settings', 'main',
    'configs.ragflow', 'configs.logger',
    'services.chat_assistant', 'services.dataset'
]

setup(
    name='iflow-mcp_oraichain_ragflow-mcp',
    version='0.1.4',
    description='Simple MCP server for RAGFlow',
    packages=find_packages(),
    python_requires='>=3.11',
    install_requires=['mcp', 'uvicorn', 'starlette', 'ragflow-sdk', 'dotenv', 'pydantic_settings'],
    py_modules=py_modules,
    package_dir={'': '.'},
    include_package_data=True
)